package es.optsicom.problem.gop.approx.experiment.paper.review;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

import es.optsicom.lib.analysis.Analysis;
import es.optsicom.lib.analysis.Analyzer;
import es.optsicom.lib.approx.ApproxMethod;
import es.optsicom.lib.approx.experiment.ApproxMethodExperiment;
import es.optsicom.lib.tablecreator.ValueProvider;
import es.optsicom.lib.util.BestMode;
import es.optsicom.lib.util.Log;
import es.optsicom.lib.util.SummarizeMode;
import es.optsicom.problem.gop.GOPInstance;
import es.optsicom.problem.gop.GOPProblem;
import es.optsicom.problem.gop.GOPSolution;
import es.optsicom.problem.gop.analysis.GapAnalyzer;
import es.optsicom.problem.gop.analysis.GapRP2;
import es.optsicom.problem.gop.approx.pr.GOPEvolutionaryPathRelinkingMethod;
import es.optsicom.problem.gop.instances.GOPRepositoryInstances;


/**
 * <p>This class runs the experimentation for the EvoPR method as defined in:</p>
 * 
 * <p><i>Path Relinking for Large Scale Global Optimization</i>, Rafael Martí, Abraham Duarte,
 * and Francisco Gortázar.</p>
 *  
 * <p>Execute this class in order to run the experimentation for 
 * the EvoPR method on all the functions. Each function is executed 25 times.</p>
 * 
 * <p>This class makes use of native resources. Include the following in the 
 * vm arguments when running it:</p>
 * 
 * <code>-Djna.library.path="lib"</code>
 * 
 * <p>This should make the vm resolve the <i>libSpecialIssue.so</i> located in the lib directory
 * that is used by the method to evaluate solutions.</p>
 * 
 * <p>The Java class representing the method is {@link GOPEvolutionaryPathRelinkingMethod}.</p>
 * 
 * @author Patxi Gortázar
 *
 */
public class EvoPRFinalExperiment {

	public static void main(String[] args) {

		GOPInstance.EVALUATIONS_LIMITED = true;
		Log.setToScreen(false);
		
		run();
	}

	public static ApproxMethod<GOPSolution, GOPInstance>[] getFinalMethod() {

		return new ApproxMethod[] {
				new GOPEvolutionaryPathRelinkingMethod()
		};
		
	}

	public static void run() {
		
		int sizes[] = new int[]{50, 100, 200, 500, 1000};
		
		for(int size : sizes) {
			run("paper", size, getFinalMethod());
		}
	}
	
	public static void run(String useCase, int size, ApproxMethod<GOPSolution, GOPInstance>... methods) {

		GOPRepositoryInstances repo = (GOPRepositoryInstances) GOPProblem.getInstance().getInstancesRepository(useCase);
		
		ApproxMethodExperiment<GOPSolution, GOPInstance> experiment = new ApproxMethodExperiment<GOPSolution, GOPInstance>(Arrays.asList(methods), 25);
		experiment.setProblem(GOPProblem.getInstance());
		experiment.setInstanceFiles(repo.getInstanceFileWithSizes(size));
		experiment.setRecordEvolution(false);
		experiment.setResultsDir(new File("Experiments", "Final_" + size));
		
		try {
			experiment.executeExperiment();
			
			GapAnalyzer analyzer = new GapAnalyzer(BestMode.MIN_IS_BEST);

			GapRP2 gapRPAvg = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
			gapRPAvg.setSummarizeMode(SummarizeMode.AVERAGE);
			GapRP2 gapRPMin = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
			gapRPMin.setSummarizeMode(SummarizeMode.MIN);
			GapAnalyzer avgAnalyzer = new GapAnalyzer(BestMode.MIN_IS_BEST, gapRPAvg, gapRPMin);

			List<Analyzer> analyzers = new ArrayList<Analyzer>();
			analyzers.add(analyzer);
			analyzers.add(avgAnalyzer);
			
			Locale.setDefault(Locale.UK);
			
			Analysis analysis = new Analysis(experiment.getResultsDir(), analyzers);
			analysis.executeAnalysis();
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
