package es.optsicom.problem.gop.approx.improvement;

import java.util.Random;

import es.optsicom.lib.approx.improvement.AbstractImprovementMethod;
import es.optsicom.lib.experiment.CurrentExperiment;
import es.optsicom.lib.util.Id;
import es.optsicom.lib.util.Log;
import es.optsicom.lib.util.RandomManager;
import es.optsicom.problem.gop.GOPInstance;
import es.optsicom.problem.gop.GOPProblem;
import es.optsicom.problem.gop.GOPSolution;
import es.optsicom.problem.gop.function.Function;

public class SimplexSearch extends AbstractImprovementMethod<GOPSolution, GOPInstance> {

	private int nfunk;
	private double alpha = 7.0;
	private double beta = 0.5;
	private double gamma = 2.0;

	@Override
	public boolean internalImproveSolution(GOPSolution solution, long millis) {
		double p[][], y[], perturb, xold;
		
		int numEvaluationsBefore = solution.getInstance().getNumEvaluations();
		
		Random random = RandomManager.getRandom();
		double value = solution.getWeight();
		Function f = solution.getInstance().getFunction();
		
		GOPSolution original = (GOPSolution) solution.createCopy();
		
		int numVar = solution.getInstance().getDimension();
		
		p = new double[numVar + 2][numVar + 1];
		y = new double[numVar + 2];

		//First point of the simplex. Solution being improved
		for(int i = 1; i <= numVar; i++){
			p[1][i] = solution.get(i - 1);
		}

		y[1] = solution.getWeight();

		for(int i = 2; i <= numVar + 1; i++){
			for(int j = 1; j <= numVar; j++){
				p[i][j] = solution.get(j - 1);
			}
		}

		for(int j = 1; j <= numVar; j++){
			xold        = solution.get(j - 1);
			double boundsFactor = 0.01 * (f.getUpperBound()-f.getLowerBound());
			perturb     = -1 + 2*(random.nextDouble());
			perturb *= boundsFactor;
			double varValue = makeMovement(xold, perturb, f.getLowerBound(), f.getUpperBound());
			if(varValue < f.getLowerBound()) {
				varValue = f.getLowerBound();
			} else if(varValue > f.getUpperBound()) {
				varValue = f.getUpperBound();
			}
			solution.set(j - 1, varValue);
			if (!solution.isFeasible()) {
				throw new Error("Unfeasible solution");
			}
			
			p[j + 1][j] = solution.get(j - 1);
			y[j + 1]    = solution.getWeight();
			solution.set(j-1, xold);
		}

		if(solution.getWeight() > original.getWeight()) {
			throw new Error("Solution is worst than original");
		}
		
		/* Call Nelder and Mead's Simplex method */
		amoeba(p, y, numVar, 0.001, solution.getInstance());

		for(int i = 1; i <= numVar + 1; i++){
			if(value > y[i]){
				for(int j = 1; j <= numVar; j++){
					solution.set(j - 1, p[i][j]);
					if(solution.get(j - 1) > f.getUpperBound()){
						solution.set(j - 1, f.getUpperBound());
					}
					if(solution.get(j - 1) < f.getLowerBound()){
						// FIXME: should be lowerBound 
						solution.set(j - 1, f.getLowerBound());
					}
				}
				value = y[i];
			}
		}
		
		if(GOPInstance.SAVE_EVALUATION_EVENTS) {
			CurrentExperiment.addEvent(GOPProblem.SIMPLEX_EVALUATIONS, solution.getInstance().getNumEvaluations() - numEvaluationsBefore);
		}
		
		if(solution.getWeight() > original.getWeight()) {
			Log.debugln("Solution is worst than original");
			for(int var = 0; var < f.getDimension(); var++) {
				solution.set(var, original.get(var));
			}
			return false;
		}

		if(!solution.isFeasible()) {
			throw new Error("Unfeasible solution");
		}

		return solution.isBetterThan(original);
	}
	
	private double makeMovement(double value, double step, double lowBound, double upBound){

		value += step;

		if(value > upBound){
			value = upBound - Math.abs(step)/2;
		}
		else if(value < lowBound){
			value = lowBound + Math.abs(step)/2;
		}
		return(value);
	}

	/* Nelder and Mead Simplex Method for Nonlinear unconstrained optimization problems          */
	private void amoeba(double p[][], double y[], int numVar, double ftol, GOPInstance instance){
		final int nmax = 1000;
		int i, j, ilo, ihi, inhi, mpts = numVar + 1;
		double ytry, ysave, sum, rtol, psum[];

		psum = new double[numVar + 1];
		nfunk = 0;

		for (j = 1; j <= numVar; j++){ 
			for (i = 1, sum = 0.0; i <= mpts; i++) {
				sum += p[i][j]; 
			}		
			psum[j]=sum; 
		}
		
		for (;;){
			ilo = 1;
			if(y[1] > y[2]) {
				ihi = 1;
				inhi = 2;
			} else {
				ihi = 2;
				inhi = 1;
			}

			for (i = 1; i <= mpts; i++) {
				if (y[i] < y[ilo]){ 
					ilo = i;
				}
				if (y[i] > y[ihi]){
					inhi = ihi;
					ihi = i;
				} 
				else if (y[i] > y[inhi]){
					if (i != ihi){ 
						inhi = i;
					}
				}
			}
			
			rtol = 2.0 * Math.abs(y[ihi] - y[ilo])/(Math.abs(y[ihi]) + Math.abs(y[ilo]));
			
			if (rtol < ftol){ 
				break;
			}
			
			if (nfunk >= nmax){ 
				// No sé si hace falta reconocer que hemos salido por aquí
				return;
			}
			
			nfunk += 2;
			
			ytry = amotry(p, y, psum, numVar, ihi, -alpha, instance);
			
			if (ytry <= y[ilo]){
				ytry = amotry(p, y, psum, numVar, ihi, gamma, instance);
			}
			else if (ytry >= y[inhi]){
				ysave = y[ihi];
				ytry = amotry(p, y, psum, numVar, ihi, beta, instance);
				if (ytry >= ysave){
					for (i = 1; i <= mpts; i++){
						if (i != ilo) {
							for (j = 1; j <= numVar; j++){
								psum[j] = 0.5 * (p[i][j] + p[ilo][j]);
								p[i][j] = psum[j];
							}
							GOPSolution newSol = new GOPSolution(instance);
							for(int var = 0; var < instance.getDimension(); var++) {
								newSol.set(var, psum[var+1]);
							}
							y[i] = newSol.getWeight();
						}
					}
					nfunk += numVar;
					for (j = 1; j <= numVar; j++){ 
						for (i = 1, sum = 0.0; i <= mpts; i++) {
							sum += p[i][j]; 
						}		
						psum[j]=sum; 
					}
				}
			}
		}
	}

	double amotry(double p[][], double y[], double psum[], int ndim, int ihi, 
			double fac, GOPInstance instance) {
		
		int j;
		double fac1, fac2, ytry, ptry[];

		ptry = new double[ndim + 1];
		fac1 = (1.0 - fac)/(double)ndim;
		fac2 = fac1 - fac;
		
		for (j = 1; j <= ndim; j++){
			ptry[j] = psum[j] * fac1 - p[ihi][j] * fac2;
		}
		
		// No comentado por mi
		//ytry = compute_error(net,ptry);
		
		GOPSolution newSol = new GOPSolution(instance);
		for(int var = 0; var < instance.getDimension(); var++) {
			newSol.set(var, ptry[var+1]);
		}
		ytry = newSol.getWeight();
		nfunk++;
		if (ytry < y[ihi]) {
			y[ihi] = ytry;
			for (j = 1; j <= ndim; j++) {
				psum[j]  += ptry[j] - p[ihi][j];
				p[ihi][j] = ptry[j];
			}
		}
		
		return ytry;
	}

	@Override
	public boolean isDeterminist() {
		return false;
	}

	@Id
	public double getAlpha() {
		return alpha;
	}

	public void setAlpha(double alpha) {
		this.alpha = alpha;
	}

	@Id
	public double getBeta() {
		return beta;
	}

	public void setBeta(double beta) {
		this.beta = beta;
	}

	@Id
	public double getGamma() {
		return gamma;
	}

	public void setGamma(double gamma) {
		this.gamma = gamma;
	}

	

}
